{% extends "base_new.html" %}

{% block title %}Invalid Login{% endblock %}

{% block content %}

  <h2>Your login details are invalid!</h2>
  <p> Click <a href="{% url "login" %}">here</a> to login again.</p>

  <p> {{ message }}</p>
{% endblock %}
